

/////////////////////////////
//      Vertex Shader      //
/////////////////////////////

// constant buffer : transformation totale
cbuffer ConstantsVS{
	float4x4	mFull;
}


struct VSin{ 
	float3 Pos : POSITION0;
	float3 N   : NORMAL0;
	
};
	
struct VSout{
	float4 PosG : POSITION0;
	float4 N    : NORMAL0;
	float4 Pos  : SV_POSITION;
	
};

VSout VSmain( VSin Vin ){
	VSout Vout = (VSout)0;
	
	float4 pos = float4(Vin.Pos, 1.0f);
	
	// transformation
	Vout.Pos = mul( pos, mFull);
	
	Vout.PosG = float4(Vin.Pos, 1.f);	// position du sommet (repere local)
	Vout.N = float4(Vin.N, 0.0f);		// normale

	return Vout;
}





	
/////////////////////////////
//      Pixel Shader      //
/////////////////////////////

// constant buffer : donnes pour eclairage
cbuffer ConstantsPS{
	float4		iObsPos;		// position observateur (local)
	float4		lAcol;			// couleur ambiante
	float4		kakdks;			// coeff : Ka, Kd, Ks
	float4		iLPos;			// position lampe (local)
	float4		lCol;			// couleur lampe
	float4		objColD;		// couleur diffuse objet
	float4		objColS;		// couleur speculaire
	float		ns;				// power
	
};


struct PSin{ 
	float4 PosG : POSITION0;
	float4 N    : NORMAL0;
};

struct PSout{
	float4	Val : SV_Target;
};

PSout PSmain( PSin Pin ){
	PSout Pout = (PSout)0;
	
	float4  Normale, L, V, H;
	float4  Ld, Ls;
	
	float4	Ka = kakdks.x,
			Kd = kakdks.y,
			Ks = kakdks.z;
	
	// calcul de la geometrie
	Normale = normalize( Pin.N );				 // interpol,  normaliser
	L = normalize( iLPos - Pin.PosG );  
	V = normalize( iObsPos - Pin.PosG );
	H = normalize( V + L );
	
	// calcul de l'eclairage
	Ld = dot( Normale, L );						 // contribution diffuse
	Ls = pow( max(dot(Normale,H),0.f) , ns );	 // contribution speculaire
	
	// couleur
	Pout.Val = Ka * lAcol + lCol * ( Kd*objColD*Ld + Ks*objColS*Ls );
	
	return Pout;
}



